(load "~/quicklisp/setup.lisp") ; need to load quicklisp
(ql:quickload '(:cl-raylib :cl-bcrypt :websocket-driver-client
			   :alexandria :com.inuoe.jzon))

(defpackage :mmo-client
  (:use :common-lisp :cl-raylib :raylib :3d-vectors
	:websocket-driver-client :alexandria :com.inuoe.jzon))

(in-package :mmo-client)

(defvar *client* (wsd:make-client "ws://localhost:12345"))
(defvar *logged-in-users* (make-hash-table))
(defvar *current-user* (make-hash-table))
(defvar *login* (make-hash-table))

(wsd:start-connection *client*)
(wsd:on :message *client*
        (lambda (message)
	  (let* ((json (com.inuoe.jzon:parse message))
		 (cmd (gethash "cmd" json))
		 (uid (gethash "uid" json)))
	  (cond
	   ((equalp cmd "login")
	      (setf (gethash "uid" *current-user*) uid))
	   ((equalp cmd "user-login")
	    (setf *logged-in-users* json))
	   ((equalp cmd "logout")
	      (remhash uid *logged-in-users*))
	   ((equalp cmd "move")
	      (setf (gethash "x" (gethash uid *logged-in-users*))
		    (gethash "x" json))
	      (setf (gethash "y" (gethash uid *logged-in-users*))
		    (gethash "y" json)))
	   (t 
	    (format t "Unknown message: ~A~%" message))))))

(defun send-move (camera username)
  (let ((move (make-hash-table)))
    (setf (gethash "cmd" move) "move")
    (setf (gethash "user" move) username)
    (setf (gethash "x" move) (vx (camera3d-target camera)))
    (setf (gethash "y" move) (vz (camera3d-target camera)))
    (wsd:send *client* (com.inuoe.jzon:stringify move))))

(let* ((screen-width 800)
       (screen-height 450)
       (username "username")
       (title "mmo client - common lisp")
       (camera (make-camera3d :position (vec 4.0 2.0 4.0)
                              :target (vec 0.0 0.5 0.0)
                              :up (vec 0.0 1.0 0.0)
                              :fovy 60.0
                              :projection :camera-perspective))
       (cube-screen-pos (vec 0.0 0.0)))

  (setf (gethash "cmd" *login*) "user-login")
  (setf (gethash "user" *login*) username)
  (setf (gethash "user" *current-user*) username)
  (wsd:send *client* (com.inuoe.jzon:stringify *login*))

  (with-window (screen-width screen-height title)
   ;; (disable-cursor)
    (set-target-fps 60) ; Set our game to run at 60 FPS
    (loop
      until (window-should-close) ; detect window close button or ESC key
      do
	 (update-camera camera :camera-third-person)

         (when (is-key-down 87)
	     (send-move camera username))
         (when (is-key-down 65)
	     (send-move camera username))
         (when (is-key-down 83)
	     (send-move camera username))
         (when (is-key-down 68)
	     (send-move camera username))

	 (setf cube-screen-pos
	       (get-world-to-screen
		(v+ (camera3d-target camera) (vec 0 1.0 0)) camera))
	 (with-drawing
	     (clear-background :raywhite)
	   (with-mode-3d (camera)
	     (draw-cube (camera3d-target camera) 1.0 1.0 1.0 :red)
	     (draw-text username
			(- (floor (vx cube-screen-pos))
			   (floor (measure-text username 20) 2)) 
			(floor (vy cube-screen-pos) ) 20 :black)

	     (loop :for user :being :the :hash-key :of *logged-in-users* :do
		   (let ((x (gethash "x" user))
			 (y (gethash "y" user)))
		     (draw-cube (vec x 0.5 y) 1.0 1.0 1.0 :red)
		     (let ((name (gethash "username" user)))
		       (draw-text name
				  (- (floor (vx cube-screen-pos))
				     (floor (measure-text name 20) 2))
				  (floor (vy cube-screen-pos)) 20 :black))))
	     (draw-grid 20 1.0))))))

(wsd:close-connection *client*)
